import shodan
import socket

class ShodanFinder :
    API = 'tedoHTagcSRqQm9LhIzUCqIqUvI06dHz'

    def __init__(self, sites):
        self.sites_list = sites
        self.data_instance = {}

    def getHost(self, site):
        return socket.gethostbyname(site)

    def extractData(self, host):
        api = shodan.Shodan(self.API)
        self.data_instance = api.host(host)

    def extractPortsList(self):
        return self.data_instance['ports']
    
    def extractUsedServer(self, port):
        for port_container in self.data_instance['data']:
            if port_container['port'] == port:
                return port_container['http']['server']

    def extractBannere(self, port):
        for port_container in self.data_instance['data']:
            if port_container['port'] == port:
                return port_container['data']

    def extractTechnologies(self, port):
        for port_container in self.data_instance['data']:
            if port_container['port'] == port:
                return port_container['http']['components'] if hasattr(port_container['http'], 'components') else "Not Found"

    def shodanProcedure(self):
        sites_services = []

        for site in self.sites_list:
            sites_services.append({'name': site})

            host = self.getHost(site)
            self.extractData(host)
            sites_services[len(sites_services) - 1]['host'] = host

            ports = self.extractPortsList()
            services = []
            for port in ports:
                services.append({
                    'port': port,
                    'serveur': self.extractUsedServer(port),
                    'banner': self.extractBannere(port),
                    'technologies': self.extractTechnologies(port)
                })
            sites_services[len(sites_services) - 1]['services'] = services

        return sites_services




from docx.shared import Pt
from docx.enum.style import WD_STYLE_TYPE
from docx import Document
from shodan_finder import ShodanFinder
from datetime import datetime

shodan_instance = ShodanFinder(['adacis.com', 'google.com'])

sites_services = shodan_instance.shodanProcedure()

doc = Document()
doc.add_heading('Module X : Shodan Finder', 0)

# Custom Style
obj_styles = doc.styles
obj_charstyle = obj_styles.add_style('Banner', WD_STYLE_TYPE.CHARACTER)
obj_font = obj_charstyle.font
obj_font.size = Pt(10)
obj_font.name = 'Montserrat'

for site_service in sites_services:
    doc.add_paragraph("•  Site Name : {}".format(site_service['name']))
    doc.add_paragraph("     >  Host : {}".format(site_service['host']))

    for service in site_service['services']:
        doc.add_paragraph("     >  Service at port : {} ".format(service['port']))
        doc.add_paragraph("         +  serveur : {} ".format(service['serveur']))
        doc.add_paragraph("         +  technologies : {} ".format(service['technologies']))
        
        doc.add_paragraph("         +  banner : ")
        doc.add_paragraph("")
        
        paragraph = doc.add_paragraph("")
        paragraph_format = paragraph.paragraph_format
        paragraph_format.line_spacing = Pt(18)
        paragraph.add_run(service['banner'], style = 'Banner')

date = datetime.now()
doc.save("results/shodan_finder_result_{}.docx".format(date.strftime("%m%d%Y%H%M%S")))



import os
import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

with open('requirements.txt') as f:
    required = f.read().splitlines()
	
setuptools.setup(
     name='shodanfinder',  
     version='0.3',
     author="KDragon",
     author_email="elkarouani@gmail.com",
     description="Get an observation on any website on internet using Shodan",
     long_description=long_description,
   long_description_content_type="text/markdown",
     url="https://github.com/elkarouani/shodan-finder-demo",
     classifiers=[
         "Programming Language :: Python :: 3",
         "License :: OSI Approved :: MIT License",
         "Operating System :: OS Independent",
     ],
	 packages=['shodanfinder'],
	 include_package_data=True,
	 zip_safe=False,
	 install_requires=required
 )


import unittest
from shodan_finder import ShodanFinder

class TestShodanFinder(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        print('Starting tests on Shodan Finder Module')
        print('Initializing instances ...')
        cls.shodan_finder_instance  = ShodanFinder(['adacis.com'])
        cls.shodan_finder_instance.extractData('52.58.78.16')

    def test_init(self):
        print('Testing constructor ...')
        self.assertEqual(self.shodan_finder_instance.sites_list, ['adacis.com'])
        self.assertEqual(type(self.shodan_finder_instance.data_instance).__name__, 'dict')

    def test_get_host(self):
        print('Testing get host method ...')
        self.assertEqual(self.shodan_finder_instance.getHost('adacis.com'), '52.58.78.16')

    def test_extract_data(self):
        print('Testing extract data method ...')
        self.assertEqual(type(self.shodan_finder_instance.data_instance).__name__, 'dict')

    def test_extract_ports_list(self):
        print('Testing extract ports list method ...')
        self.assertEqual(self.shodan_finder_instance.extractPortsList(), [80, 443])

    def test_extract_used_server(self):
        print('Testing extract used server method ...')
        self.assertEqual(self.shodan_finder_instance.extractUsedServer(80), "openresty/1.13.6.2")

    def test_extract_bannere(self):
        print('Testing extract bannere method ...')
        self.assertEqual(type(self.shodan_finder_instance.extractBannere(80)).__name__, "str")
    
    def test_extract_technologies(self):
        print('Testing extract technologies method ...')
        self.assertEqual(type(self.shodan_finder_instance.extractTechnologies(80)).__name__, "str")

    def test_shodan_procedure(self):
        print('Testing shodan procedure method ...')
        self.assertEqual(type(self.shodan_finder_instance.shodanProcedure()).__name__, "list")

if __name__ == "__main__":
    unittest.main()

